==> 1.0.0.tar.gz: lib/src/lists/FormalParameterList.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/FormalParameters.dart';

/*
<formalParameterList> ::= '(' ')'
| '(' <normalFormalParameters> ','? ')'
| '(' <normalFormalParameters> ',' <optionalFormalParameters> ')'
| '(' <optionalFormalParameters> ')'

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 21
*/
class FormalParameterList extends ParserType {
  int caseNum;
  NormalFormalParameters normalFormalParameters;
  OptionalFormalParameters optionalFormalParameters;
  bool hasComma;

  FormalParameterList(Lexer lexer) {
    hasComma = false;
    int attempt, maxAttempts = 4;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
          case 2: _case2(lexer); break;
          case 3: _case3(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in FormalParameterList: ALL FAILED');
  }
  
  /* '(' ')' */
  void _case0(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();
    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 0;
  }
  

  /* '(' <normalFormalParameters> ','? ')' */
  void _case1(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    normalFormalParameters = NormalFormalParameters(lexer);

    if (isType(lexer, TType.COMMA)) {
      hasComma = true;
      lexer.advance();
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 1;
  }
  
  /* '(' <normalFormalParameters> ',' <optionalFormalParameters> ')' */
  void _case2(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    normalFormalParameters = NormalFormalParameters(lexer);

    checkType(lexer, TType.COMMA);
    lexer.advance();

    optionalFormalParameters = OptionalFormalParameters(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 2;
  }
  
  /* '(' <optionalFormalParameters> ')' */
  void _case3(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    optionalFormalParameters = OptionalFormalParameters(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 3;
  }
  
  String toString() {
    switch (caseNum) {
      case 0: return '()'; break;
      case 1: return '($normalFormalParameters' + (hasComma ? ',' : '') + ')'; break;
      case 2: return '($normalFormalParameters, $optionalFormalParameters)'; break;
      case 3: return '($optionalFormalParameters)'; break;
    }
    return '';
  }
  
}

==> 1.1.1.tar.gz: lib/src/lists/FormalParameterList.dart <==
import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/FormalParameters.dart';

/// <formalParameterList> ::= '(' ')'
/// | '(' <normalFormalParameters> ','? ')'
/// | '(' <normalFormalParameters> ',' <optionalFormalParameters> ')'
/// | '(' <optionalFormalParameters> ')'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 21
class FormalParameterList extends ParserType {
  int caseNum;
  NormalFormalParameters normalFormalParameters;
  OptionalFormalParameters optionalFormalParameters;
  bool hasComma;

  FormalParameterList(Lexer lexer) {
    hasComma = false;
    int attempt, maxAttempts = 4;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
          case 2:
            _case2(lexer);
            break;
          case 3:
            _case3(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType(
          'Tried all options in FormalParameterList: ALL FAILED');
    }
  }

  /// '(' ')'
  void _case0(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();
    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 0;
  }

  /// '(' <normalFormalParameters> ','? ')'
  void _case1(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    normalFormalParameters = NormalFormalParameters(lexer);

    if (isType(lexer, TType.COMMA)) {
      hasComma = true;
      lexer.advance();
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 1;
  }

  /// '(' <normalFormalParameters> ',' <optionalFormalParameters> ')'
  void _case2(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    normalFormalParameters = NormalFormalParameters(lexer);

    checkType(lexer, TType.COMMA);
    lexer.advance();

    optionalFormalParameters = OptionalFormalParameters(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 2;
  }

  /// '(' <optionalFormalParameters> ')'
  void _case3(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    optionalFormalParameters = OptionalFormalParameters(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
    caseNum = 3;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '()';
        break;
      case 1:
        return '($normalFormalParameters' + (hasComma ? ',' : '') + ')';
        break;
      case 2:
        return '($normalFormalParameters, $optionalFormalParameters)';
        break;
      case 3:
        return '($optionalFormalParameters)';
        break;
    }
    return '';
  }
}
